isLive = true;

protocol = "https://";
host = isLive? "www.holidaylettings" : "dev30.hq.internalhl";
defaultDomain = isLive? ".co.uk" : ".com"; // this should be .co.uk for live but dev server needs .com
shouldBuildUrl = typeof forcedUrl === 'undefined';
if(shouldBuildUrl){
    baseUrl = protocol + host + defaultDomain;
}
else{
    baseUrl = forcedUrl;    
}


inboxPath = "/content/intl_inbox/inbox";
conversationPath = "/content/conversation/conversation/";
getPosPath = "/content/list_your_home/pos_for_token";

tokenKey = "ownerAppToken";
baseUrlKey = "ownerAppBaseUrl";
langKey = "ownerAppLang";
             
isConnected = null;

// set by a push notification message - don't redirect to other pages when this is set
redirectingToConversationPage = false;

/*
 * the device type will be set ondeviceready - initialize to 0 (unknown)
 * This corresponds to the C# enum in the hlmain repository, 
 * HolidayLettings.Data/Bookings/Notifications/PhoneType.cs
 */ 
deviceType = 0;

// the token used for identifying the user for push notifications
pushToken = "";

// if we can't get a token, or if the user declines push notifications, we still 
// need to give a token for the purpose of remaining logged in - we'll make a random
// token for them and prefix with this string to signify that it should not be used 
// for actual push notifications
noTokenPrefix = "no_token";

/*
 * This corresponds to the C# enum in hlmain repository
 * HolidayLettings.Data/Bookings/Notifications/PhoneType.cs
 */
deviceTypeMap = {
	"ios" : 1,
	"android" : 2
};

/*
 * A map from language abbreviation to the domain for that language
 */
langToDomainMap = {
	"en" : defaultDomain,
	"es" : ".es",
	"fr" : ".fr",
	"it" : ".it",
    "de" : ".de",
    "nl" : ".nl",
    "pt" : ".com.br",
    "se" : ".se",
    "tr" : ".com.tr",
    "gr" : ".gr"
};

googleResponse = "";

logMessage = function(message) {
	console.log(message);
}; 

saveLang = function(lang) {
    if(lang && lang.length > 2) {
        lang = lang.substr(0, 2);
    }
    language = lang;
    window.localStorage.setItem(langKey, language);
};

setLang = function() {
    language = window.localStorage.getItem(langKey);    
    if(!language) {
        language = navigator.language;
        if(language.length > 2) { 
            language = language.substr(0, 2);    
        }
        saveLang(language);
    }
};

updateLangStrings = function() {
    document.getElementById('loadingText').innerHTML = getTranslation("loading", language);    
    document.getElementById('greenBtn').innerHTML = getTranslation("reload", language);
    document.getElementById('offlineText').innerHTML = getTranslation("offline", language);
    document.getElementById('tryAgainText').innerHTML = getTranslation("offlineMsg", language);
};

setBaseUrl = function() {
    if(shouldBuildUrl){
        baseUrl = window.localStorage.getItem(baseUrlKey);
        if(!baseUrl) {
            var domain = defaultDomain;
            var mappedDomain = langToDomainMap[language];
            if(mappedDomain) domain = mappedDomain;
            baseUrl = protocol + host + domain;
        }
    }
};

hl = {};
hl.modules = {};
hl.modules.ownerApp = {};

setDeviceType = function() {
	var deviceTypeName = device.platform.toLowerCase();
    deviceType = deviceTypeMap[deviceTypeName];
    if(!deviceType) deviceType = 0; // stands for unknown;
};
	
// Android handler
hl.modules.ownerApp.onNotificationGCM = function(e) {
    logMessage('gcm-message in notification callback with argument: ' + JSON.stringify(e));

	if(e.event == 'registered' && e.regid && e.regid.length > 0) {
    	saveToken(e.regid);
        setPosAndRedirect();
	}
	else if(e.event == 'message') {
        redirectingToConversationPage = true;
        redirectToConversationPage(e.payload.conversationId, e.payload.reason);
	}
};
  
// iOS handler
hl.modules.ownerApp.onNotificationAPN = function(event) {
    // If we need to show an alert, we have two options. Either use the JS alert()
    // or we can include the org.apache.cordova.dialogs plugin and use 
    // navigator.notification.alert()
    // if(event.alert) {
    //    alert(event.alert);
    //}
    // We need to determine the right time to show/create a badge. CONVERSION-716
    //if(event.badge) {
    //    window.plugins.pushNotification.setApplicationIconBadgeNumber(successHandler, function(){},  event.badge);
    //}
    if(event.conversationId) {
        redirectingToConversationPage = true;
        redirectToConversationPage(event.conversationId, event.reason);
    }
    logMessage("ios notification received event: " + JSON.stringify(event)); 
};

// called after a successful android push registration - onNotificationGCM is called with the actual token,
// here the result is just 'OK', so this doesn't seem to be very useful...
successHandler = function(result) {
    googleResponse = result;
	logMessage("successHandler result: " + result);
};

// called after an error is encountered registering for push notifications for either iOS or android
errorHandler = function(error) {
	logMessage("errorHandler error: " + error);
    setPosAndRedirect();
};

// called after a successful iOS push registration, the argument is the actual token, so the work is done here
tokenHandler = function(token) {
    logMessage("token handler received token: " + token);
	saveToken(token);
    setPosAndRedirect();
};

registerForPushToken = function() {
	var pushNotification = window.plugins.pushNotification;
	logMessage("register for push with "+pushNotification);
	if (deviceType === 2) { // Android
    	pushNotification.register(successHandler, errorHandler, {"senderID":"678597945675","ecb":"hl.modules.ownerApp.onNotificationGCM"});
    }
    else if (deviceType === 1) { // ios
    	pushNotification.register(tokenHandler, errorHandler, {"badge":"true","sound":"true","alert":"true","ecb":"hl.modules.ownerApp.onNotificationAPN"});
    }	
    else { // unrecognized device type, just head over to the login page. Push notifications won't work. This shouldn't happen
    	setPosAndRedirect();
    }
};

saveToken = function(token) {
    pushToken = token;
    window.localStorage.setItem(tokenKey, token);
};

redirectToInbox = function() {
	var url = baseUrl + inboxPath + getAppUrlParams();
	redirectToUrl(url);
};

redirectToConversationPage = function(conversationId, reason) {
    logMessage("attempting to redirect to conversation page " + conversationId);
    var callback = function() {
        var pushParams = '&pushSrc=out&reason=' + reason;
        var url = baseUrl + conversationPath + getAppUrlParams() + "&cid=" + conversationId + pushParams + "#blueHeader";
        redirectToUrl(url);
    };
    getPosForToken(callback, "redirectToConversationPage");
};

redirectToUrl = function(url) {

    if(redirectingToConversationPage && url.indexOf(conversationPath) == -1) {
        // we're waiting to redirect to the conversation page, don't redirect to any other page
        return;
    }

    document.getElementById('loadingDisplay').style.display = 'none';
    logMessage("about to redirect to " + url);
    setTimeout(function() {
        if(redirectingToConversationPage && url.indexOf(conversationPath) == -1) {
            // we're waiting to redirect to the conversation page, don't redirect to any other page
            return;
        }
        document.location.replace(url);
    }, 100);
};

getAppUrlParams = function(omitQuestionMark) {
    if(pushToken === "") {
        // use saved token from previous app launch if we couldn't register again
        pushToken = window.localStorage.getItem(tokenKey);
        if(!pushToken) {
            // still no token? generate a random token, this will be used to identify this user as an app user and
            // keep their session active, but will obviously not actually do anything for push notifications
            saveToken(generateNoToken());
        }
    }
	return (omitQuestionMark? "": "?") + "at=" + appType + "&dt=" + deviceType + "&pt=" + pushToken;
};

generateNoToken = function() {
    return noTokenPrefix + getRandomString() + getRandomString();   
};

getRandomString = function() {
    var randStr = Math.random().toString(); // e.g. "0.10901349384873948"
    return randStr.substr(2, randStr.length); // removes the "0." to give a random string of numbers
};

setPosAndRedirect = function() {
    if(redirectingToConversationPage) return;

    var callback = function() {
        if(redirectingToConversationPage) return;
        redirectToInbox();
    };
    if(shouldBuildUrl){
        getPosForToken(callback, "setPosAndRedirect");
    }
    else{
        callback();
    }
};

/**
 * Makes an ajax call to find the pos for the owner associated with the current app params
 * @param callback - a function which will take the retrieved be run after retrieving the pos
 * and setting the domain. This callback will redirect to the proper page using the proper domain
 */
getPosForToken = function(callback, context) {
	var request = new XMLHttpRequest();
	var url = baseUrl + getPosPath;
	logMessage("context = " + context + " requesting url: " + url);
	var params = getAppUrlParams(true);
	logMessage("with params: " + params);
    request.open('POST', url);
    
    //Send the proper header information along with the request
	request.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
	request.setRequestHeader("Content-length", params.length);
	request.setRequestHeader("Connection", "close");
        	
	request.onreadystatechange = function() {
		if(request.readyState == 4) { // request is done
    		if(request.status == 200) {
    			logMessage("got response json: " + request.responseText);
        		if(request.responseText && request.responseText.length > 0) {
        			var json = JSON.parse(request.responseText);
        			if(json && json.pos_url) {
        				// update the base url
        				baseUrl = protocol + (isLive? "www." : "") + json.pos_url;
                        window.localStorage.setItem(baseUrlKey, baseUrl);
        			}
                    else if(json && json.pos_correct) {
                        logMessage("context = " + context + " already on correct pos");
                    }
        			else {
        				logMessage("context = " + context + " received failing json: " + request.response);
        			}
        		}
        		else {
        			logMessage("context = " + context + " unsuccessful response: " + request.response);
        		}
        	}
        	else {
        		logMessage("context = " + context + " bad status: " + request.status);
        	}
        	callback();
        }
	};
    request.send(params);
};

setConnected = function() {
    var request = new XMLHttpRequest();
    var url = baseUrl + getPosPath;
    var params = "t=1";
    request.open('POST', url, false);
    logMessage("about to test connectivity to url " + url);
    
    //Send the proper header information along with the request
    request.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
    request.setRequestHeader("Content-length", params.length);
    request.setRequestHeader("Connection", "close");
    
    try { 
        setTimeout(function() { if(isConnected === null) setOffline();}, 1500);
        request.send(params);
        if(request.status >= 200 && request.status < 304) {
            logMessage("connection to HL confirmed");
            isConnected = true;
            return true;
        }
    }
    catch(e) { logMessage("error trying to test connectivity: " + e); }
    setOffline();
    return false;
};

setOffline = function() {
    isConnected = false; 
    document.getElementById('loadingDisplay').style.display = 'none';
    document.getElementById('offlineDisplay').style.display = 'block';   
};

setOnline = function() {
    isConnected = true; 
    document.getElementById('offlineDisplay').style.display = 'none';
    document.getElementById('loadingDisplay').style.display = 'block';
};

reloadBtnClick = function() {
    setOnline();
    setTimeout(run, 1000);
};

run = function() {
    document.getElementById('loadingDisplay').style.display = 'block';
    document.getElementById('offlineDisplay').style.display = 'none';
    setLang();
    updateLangStrings();
    setBaseUrl();
    setDeviceType();
    setConnected();
    // the call back functions from registering will redirect to proper page
    if(isConnected) {
        registerForPushToken();
        setTimeout(function() {
            if(googleResponse=="OK"){
                logMessage("Waited for 4s. Keep waiting, google gave us the OK");
            } else{
                logMessage("Waited for 4s. Redirect, something is taking too long.");
                setPosAndRedirect();
            }
        }, 4000);
    }
};

document.addEventListener("deviceready", run, false);
document.addEventListener("resume", run, false);
